local ADDON_NAME, namespace = ...;
local functions = {};
local loadFrame = CreateFrame("FRAME");
local llrConfigWindow = CreateFrame("Frame", "llr", UIParent);
local llrLoaded = false;

--Add config to standard wow interface window.
llrConfigWindow.name = "LPT Loot Roll";
InterfaceOptions_AddCategory(llrConfigWindow);

--Check button factory.
local function createCheckbutton(name, parent, x_loc, y_loc, displayname)
	local checkbutton = CreateFrame("CheckButton", name, parent, "ChatConfigCheckButtonTemplate");
	checkbutton:SetPoint("TOPLEFT", x_loc, y_loc);
	getglobal(checkbutton:GetName() .. 'Text'):SetText(displayname);

	return checkbutton;
end

--Function for creating the roll setting frames.
local function editBoxFactory(frame, frameEditbox, x, y, title, linkedFrame)
	frame:SetPoint("TOPLEFT", llrConfigWindow, "TOPLEFT", x, y);
	frame:SetSize(100,200);
	
	--Create and set title of the setting.
	frame.title = frame:CreateFontString(nil,"OVERLAY");
	frame.title:SetFontObject(functions.fontHighlightObject);
	frame.title:SetPoint("CENTER", frame, "CENTER", 0, 5);
	frame.title:SetText(title);

	--Create the editbox for the setting.
	frame.editBox = frameEditbox;
	frame.editBox:SetPoint("CENTER", frame, "CENTER", 0, -15);
	frame.editBox:SetSize(40,40);
	frame.editBox:SetAutoFocus(false);
	frame.editBox:SetNumeric(true);
	frame.editBox:SetMaxLetters(4);
	frame.editBox:SetFontObject(functions.fontHighlightObject);
	frame.editBox:SetScript("OnKeyDown", function (self, key)
		if key == "TAB" then
			linkedFrame.editBox:SetFocus();
		end
	end);
end

--Create checkboxes.
llrConfigWindow.checkBox1 = createCheckbutton("leaderMode", llrConfigWindow, 12, -30, "Toggle Leader Mode");
llrConfigWindow.checkBox1.tooltip = "If checked, leader view is enabled.";

llrConfigWindow.checkBox2 = createCheckbutton("scrollMode", llrConfigWindow, 12, -60, "Reverse list direction");
llrConfigWindow.checkBox2.tooltip = "If checked, lists will have highest values at top and lowest at bottom.";

llrConfigWindow.checkBox3 = createCheckbutton("saveMode", llrConfigWindow, 12, -90, "Enable session memory.");
llrConfigWindow.checkBox3.tooltip = "If checked, roll history will be kept between sessions.";

--Create frames to hold the roll settings.
llrConfigWindow.mainSettingFrame = CreateFrame("Frame", nil, llrConfigWindow);
llrConfigWindow.offSettingFrame = CreateFrame("Frame", nil, llrConfigWindow);
llrConfigWindow.mogSettingFrame = CreateFrame("Frame", nil, llrConfigWindow);
llrConfigWindow.historySettingFrame = CreateFrame("Frame", nil, llrConfigWindow);

--Create the editBox frames.
llrConfigWindow.mainSettingFrame.editBox = CreateFrame("EditBox", nil, llrConfigWindow.mainSettingFrame, "InputBoxTemplate");
llrConfigWindow.offSettingFrame.editBox = CreateFrame("EditBox", nil, llrConfigWindow.offSettingFrame, "InputBoxTemplate");
llrConfigWindow.mogSettingFrame.editBox = CreateFrame("EditBox", nil, llrConfigWindow.mogSettingFrame, "InputBoxTemplate");
llrConfigWindow.historySettingFrame.editBox = CreateFrame("EditBox", nil, llrConfigWindow.historySettingFrame, "InputBoxTemplate");

--Create a sub-namespace for the global function.
namespace.functions = functions;

--Updating the config windows values.
local function updateConfigWindow()
	if not llrLoaded then return end;
	llrConfigWindow.mainSettingFrame.editBox:SetNumber(llrSettings.mainRoll);
	llrConfigWindow.offSettingFrame.editBox:SetNumber(llrSettings.offRoll);
	llrConfigWindow.mogSettingFrame.editBox:SetNumber(llrSettings.mogRoll);
	llrConfigWindow.historySettingFrame.editBox:SetNumber(llrSettings.historyLength);
	_G["leaderMode"]:SetChecked(llrSettings.leadMode);
	_G["scrollMode"]:SetChecked(llrSettings.scrollMode);
	_G["saveMode"]:SetChecked(llrSettings.saveMode);
end

--Create a "global" config toggle function so the settings window can be opened from the main lua file code.
function functions.toggleLlrConfig()
	InterfaceOptionsFrame_OpenToCategory(llrConfigWindow.name);
	InterfaceOptionsFrame_OpenToCategory(llrConfigWindow.name);
end

--Function for splitting strings.
function functions.stringSplit(inputstr, sep)
	if sep == nil then
		sep = "%s";
	end
	local t={} ; i=1
	for str in string.gmatch(inputstr, "([^"..sep.."]+)") do
			t[i] = str;
			i = i + 1;
	end
	return t;
end

--If the scale is larger than 1200 then take parentScale into consideration, otherwise just divide with scale. 
function functions.calculateScale()
	local newScale, yResolution, parentScale = 1, string.match( GetCVar( "gxWindowedResolution" ), "%d+x(%d+)" ), UIParent:GetScale();
	if tonumber(yResolution) > 1200 then
		newScale = 768/yResolution/parentScale;
	else
		newScale = 768/yResolution;
	end
	return newScale;
end

functions.fontNormalObject = CreateFont("LLR_NORMAL_FONT");
functions.fontNormalObject:CopyFontObject("GameFontNormal");
functions.fontNormalObject:SetFont("Fonts\\ARIALN.ttf", 11);

functions.fontHighlightObject = CreateFont("LLR_HIGHLIGHT_FONT");
functions.fontHighlightObject:CopyFontObject("GameFontHighLightLarge");
functions.fontHighlightObject:SetFont("Fonts\\ARIALN.ttf", 11);

--Function for saving data from config window and closing it.
local function saveData()
	if not llrLoaded then return end;
	
	local leadMode   	= llrConfigWindow.checkBox1:GetChecked();
	local scrollMode   	= llrConfigWindow.checkBox2:GetChecked();
	local saveMode   	= llrConfigWindow.checkBox3:GetChecked();
	local mainRoll   	= llrConfigWindow.mainSettingFrame.editBox:GetNumber();
	local offRoll   	= llrConfigWindow.offSettingFrame.editBox:GetNumber();
	local mogRoll 	  	= llrConfigWindow.mogSettingFrame.editBox:GetNumber();
	local historyLength = llrConfigWindow.historySettingFrame.editBox:GetNumber();
	
	if
		mainRoll 	  == offRoll or 
		mainRoll 	  == mogRoll or 
		offRoll  	  == mogRoll or 
		mainRoll 	  == 0 or
		offRoll  	  == 0 or
		mogRoll 	  == 0 or
		historyLength == 0
	then
		return;
	end
	
	llrSettings.leadMode 	  = leadMode;
	llrSettings.scrollMode 	  = scrollMode;
	llrSettings.saveMode 	  = saveMode;
	llrSettings.mainRoll 	  = mainRoll;
	llrSettings.offRoll 	  = offRoll;
	llrSettings.mogRoll 	  = mogRoll;
	llrSettings.historyLength = historyLength;
	
	functions.rollHistory(saveMode);
end

--Function for sharing user's roll settings.
local function shareSettings()
	C_ChatInfo.SendAddonMessage("lptlootroll", llrSettings.mainRoll .. "-" .. llrSettings.offRoll .. "-" .. llrSettings.mogRoll, "RAID");
end

local function setValuesToDefault()
	llrConfigWindow.mainSettingFrame.editBox = 100;
	llrConfigWindow.offSettingFrame.editBox = 10;
	llrConfigWindow.mogSettingFrame.editBox = 6;
	llrConfigWindow.historySettingFrame.editBox = 20;
	_G["leaderMode"]:SetChecked(false);
	_G["scrollMode"]:SetChecked(false);
	_G["saveMode"]:SetChecked(false);
end

--Set valueis for the config window.
llrConfigWindow:SetScale(functions.calculateScale());
llrConfigWindow:SetScript("OnShow", function(self) loadFrame:RegisterEvent("CHAT_MSG_ADDON") updateConfigWindow() end);
llrConfigWindow:SetScript("OnHide", function(self) loadFrame:UnregisterEvent("CHAT_MSG_ADDON") end);
llrConfigWindow.default = function(self) setValuesToDefault() end;
llrConfigWindow.okay = function(self) saveData() end;
llrConfigWindow:Hide();

--Create and set title of the setting.
llrConfigWindow.title = llrConfigWindow:CreateFontString(nil,"OVERLAY");
llrConfigWindow.title:SetFontObject("GameFontNormalLarge");
llrConfigWindow.title:SetPoint("TOPLEFT", llrConfigWindow, "TOPLEFT", 15, -5);
llrConfigWindow.title:SetText("LPT Loot Roll");

--Initialize the roll setting frames.
editBoxFactory(llrConfigWindow.mainSettingFrame, llrConfigWindow.mainSettingFrame.editBox, 20, -40, "Main Roll Max Value:",llrConfigWindow.offSettingFrame);
editBoxFactory(llrConfigWindow.offSettingFrame, llrConfigWindow.offSettingFrame.editBox, 20, -80, "Off Roll Max Value:",llrConfigWindow.mogSettingFrame);
editBoxFactory(llrConfigWindow.mogSettingFrame, llrConfigWindow.mogSettingFrame.editBox, 20, -120, "Mog Roll Max Value:",llrConfigWindow.historySettingFrame);

--Initialize the history length setting editbox.
editBoxFactory(llrConfigWindow.historySettingFrame, llrConfigWindow.historySettingFrame.editBox, 20, -160, "Max length of history:",llrConfigWindow.mainSettingFrame);

--Edit history editbox to have a max of two digits.
llrConfigWindow.historySettingFrame.editBox:SetMaxLetters(2);

--Create the share button.
llrConfigWindow.shareButton = CreateFrame("Button","llr_config_save", llrConfigWindow, "GameMenuButtonTemplate");
llrConfigWindow.shareButton:SetPoint("BOTTOMLEFT", llrConfigWindow, "BOTTOMLEFT", 20, 20);
llrConfigWindow.shareButton:SetSize(60,30);
llrConfigWindow.shareButton:SetText("Share");
llrConfigWindow.shareButton:SetNormalFontObject(functions.fontNormalObject);
llrConfigWindow.shareButton:SetHighlightFontObject(functions.fontHighlightObject);
llrConfigWindow.shareButton:SetScript("OnClick", shareSettings);

--Pop up for accepting settings rolls.
StaticPopupDialogs["leaderSettings"] = {
	text = "Use settings from %s?",
	button1 = "Accept",
	button2 = "Cancel",
	OnAccept = function(self, message)
		--If we accept the data, parse it to numbers and store it in settings.
		local data = functions.stringSplit(message, "-");
		local main = tonumber(data[1]);
		local off = tonumber(data[2]);
		local mog = tonumber(data[3]);
		llrSettings.mainRoll = main;
		llrSettings.offRoll = off;
		llrSettings.mogRoll = mog;
		updateConfigWindow();
	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1
};

--Create frame for handling reading and writing of data between sessions.
loadFrame:RegisterEvent("ADDON_LOADED");

loadFrame:SetScript("OnEvent", function(self, event, ...)
	local arg1,arg2,arg3,arg4 = ...;
	if event == "ADDON_LOADED" and arg1 == "LPT Loot Roll" then
		if llrSettings == nil then 
			llrSettings = {};
		end
		
		if llrSettings.raidLeadWindowPos == nil then
			llrSettings.raidLeadWindowPos = {["point"] = "CENTER", ["relativePoint"] = "CENTER", ["xOfs"] = 0, ["yOfs"] = 0};
		end
		
		if llrSettings.userWindowPos == nil then
			llrSettings.userWindowPos = {["point"] = "CENTER", ["relativePoint"] = "CENTER", ["xOfs"] = 0, ["yOfs"] = 0};
		end
		
		if llrSettings.leadMode == nil then
			llrSettings.leadMode = false;
		end
		
		if llrSettings.scrollMode == nil then
			llrSettings.scrollMode = false;
		end
		
		if llrSettings.saveMode == nil then
			llrSettings.saveMode = false;
		end
		
		if llrSettings.mainRoll == nil then
			llrSettings.mainRoll = 100;
		end
		
		if llrSettings.offRoll == nil then
			llrSettings.offRoll = 50;
		end
		
		if llrSettings.mogRoll == nil then
			llrSettings.mogRoll = 6;
		end
		
		if llrSettings.historyLength == nil then
			llrSettings.historyLength = 20;
		end
		
		if rollHistoryDB == nil then 
			rollHistoryDB = {};
		end
		
		if rollHistoryDB.rollHistory == nil then
			rollHistoryDB.rollHistory = {};
		end
		
		llrLoaded = true; 
	elseif event == "CHAT_MSG_ADDON" then
		--Check if addon message is from my addon.
		if (arg1 ~= "lptlootroll") then 
			return;
		end
		
		--Get the name of the sender, then display a pop up with said name and give it the data received.
		local name = functions.stringSplit(arg4, "-")[1];
		if name ~= UnitName("player") then 
			local dialog = StaticPopup_Show("leaderSettings", name);
			if(dialog) then
				dialog.data = arg2;
			end
		end
	end
end);