local ADDON_NAME, namespace = ...
local functions = namespace.functions;

--Initalize the main frames.
local userConfig = CreateFrame("Frame", "LootRollUser", nil, "BasicFrameTemplateWithInset");
local raidLeadConfig = CreateFrame("Frame","LootRollLeader", nil, "BasicFrameTemplateWithInset");
local llr_EventFrame = CreateFrame("Frame");

--Initalize history list and history index.
local historyIndex = 1;
local rollHistory = {};

function functions.rollHistory(bool)
	if bool then 
		rollHistory = rollHistoryDB.rollHistory;
	else
		rollHistory = {};
		rollHistoryDB.rollHistory = {};
	end
end

--Create a loading frame to get global variables.
local loadFrame = CreateFrame("FRAME"); -- Need a frame to respond to events
loadFrame:RegisterEvent("ADDON_LOADED"); -- Fired when saved variables are loaded
loadFrame:RegisterEvent("PLAYER_LOGOUT"); -- Fired when about to log out

function loadFrame:OnEvent(event, arg1)
	if event == "ADDON_LOADED" and arg1 == "LPT Loot Roll" then
		raidLeadConfig:SetPoint(llrSettings.raidLeadWindowPos.point, UIParent, llrSettings.raidLeadWindowPos.relativePoint, llrSettings.raidLeadWindowPos.xOfs, llrSettings.raidLeadWindowPos.yOfs);
		userConfig:SetPoint(llrSettings.userWindowPos.point, UIParent, llrSettings.userWindowPos.relativePoint, llrSettings.userWindowPos.xOfs, llrSettings.userWindowPos.yOfs);
		
		if llrSettings.saveMode then
			rollHistory = rollHistoryDB.rollHistory;
		end
	elseif event == "PLAYER_LOGOUT" then
		llrSettings.raidLeadWindowPos.point, _, llrSettings.raidLeadWindowPos.relativePoint, llrSettings.raidLeadWindowPos.xOfs, llrSettings.raidLeadWindowPos.yOfs = raidLeadConfig:GetPoint();
		llrSettings.userWindowPos.point, _, llrSettings.userWindowPos.relativePoint, llrSettings.userWindowPos.xOfs, llrSettings.userWindowPos.yOfs = userConfig:GetPoint();
	end
end
loadFrame:SetScript("OnEvent", loadFrame.OnEvent);

--Get random string with length of X characters for testing purposes. 
local function randomString(length)
	local res = "";
	for i = 1, length do
		res = res .. string.char(math.random(97, 122));
	end
	return res;
end

--Toggle buttons for frame.
local function toggleButtons(frame, enabled)
	if(enabled) then
		frame.mainSpeccButton:Enable();
		frame.offSpeccButton:Enable();
		frame.mogButton:Enable();
		frame.otherButton:Enable();
	else
		frame.mainSpeccButton:Disable();
		frame.offSpeccButton:Disable();
		frame.mogButton:Disable();
		frame.otherButton:Disable();
	end
end

--Setting the item in the itembutton. 
local function setItemButtonData(frame, id, link)
	frame:SetScript("OnMouseDown", function(self,button) if(IsShiftKeyDown() and button=="RightButton") then OpenAzeriteEmpoweredItemUIFromLink(link) elseif (IsControlKeyDown() and button=="LeftButton") then DressUpItemLink(link) end end);
	frame:SetNormalTexture(GetItemIcon(id));
	frame:SetScript("OnEnter", function(self) 
		GameTooltip:SetOwner(self, "ANCHOR_CURSOR"); 
		GameTooltip:ClearLines(); 
		GameTooltip:SetHyperlink(link); 
		GameTooltip:Show();
	end);
end

--Function for doing a full copy of a table.
local function deepcopy(orig)
    local orig_type = type(orig);
    local copy;
    if orig_type == 'table' then
        copy = {};
        for orig_key, orig_value in next, orig, nil do
            copy[deepcopy(orig_key)] = deepcopy(orig_value);
        end
        setmetatable(copy, deepcopy(getmetatable(orig)));
    else -- number, string, boolean, etc
        copy = orig;
    end
    return copy;
end

--Function for clearing a list and a frame.
local function clearList(frame)
	--Clear list views to prepare for new item.
	frame:Clear();
	
	--Reset scrollbar values. 
	frame:GetParent().scrollBar:SetMinMaxValues(0,0);
	frame:GetParent().scrollBar:SetValue(select(1, frame:GetParent().scrollBar:GetMinMaxValues()));
end

--Function for controlling length of history array.
local function historyLengthControl()
	if(# rollHistory >= llrSettings.historyLength+1) then
		local historyDifference = #rollHistory - llrSettings.historyLength;
		if (historyDifference > 0) then
			for i=llrSettings.historyLength+1, #rollHistory do 
				rollHistory[i]=nil 
			end;
		end
	end
end

--Function for clearing all lists and tables.
local function clearMessageFrames()
	clearList(raidLeadConfig.mainFrame.messageFrame);
	clearList(raidLeadConfig.offFrame.messageFrame);
	clearList(raidLeadConfig.mogFrame.messageFrame);
	clearList(raidLeadConfig.otherFrame.messageFrame);
end

--Update the names of the lists to reflect their values.
local function updateListNames()
	raidLeadConfig.mainFrame.title:SetText("Main (Max: " .. llrSettings.mainRoll .. ")");
	raidLeadConfig.offFrame.title:SetText("Off (Max: " .. llrSettings.offRoll .. ")");
	raidLeadConfig.mogFrame.title:SetText("Mog (Max: " .. llrSettings.mogRoll .. ")");
end

--Reverse array function 
local function reverseArray (arr)
	local i, j = 1, #arr

	while i < j do
		arr[i], arr[j] = arr[j], arr[i]

		i = i + 1
		j = j - 1
	end
end

--Add array to frame function.
local function addArrayToFrame(frame, array)
	for i,v in ipairs(array) do 
		frame:AddMessage("(" .. array[i][2] .. ") " .. array[i][1], array[i][3], array[i][4], array[i][5]);
	end
end

--Function for updating the message frames.
local function updateMessageframe(frame, array)
	frame:Clear();
	if array == nil then return end;
	local arrLength = # array;
	
	if arrLength > 7 then 
		frame:GetParent().scrollBar:SetMinMaxValues(0, arrLength-7);
	else
		frame:GetParent().scrollBar:SetMinMaxValues(0,0);
	end

	frame:SetMaxLines(arrLength);
	
	--Sort table based primarly by roll number, secondly by name.
	table.sort(array, function(a,b)
		if (a[2] > b[2]) then
			-- primary sort on roll -> a before b
			return true;
		elseif (a[2] < b[2]) then
			-- primary sort on roll -> b before a
			return false;
		else
			-- primary sort tied, resolve with secondary sort on name
			return a[1] < b[1];
		end
	end);
	
	if arrLength > 0 then
		if(llrSettings.scrollMode) then
			frame:SetInsertMode("top");
			frame:Clear();
			addArrayToFrame(frame, array);
			frame:GetParent().scrollBar:SetValue(select(1, frame:GetParent().scrollBar:GetMinMaxValues()));
		else
			frame:SetInsertMode("bottom");
			frame:Clear();
			reverseArray(array);
			addArrayToFrame(frame, array);
			frame:GetParent().scrollBar:SetValue(select(2, frame:GetParent().scrollBar:GetMinMaxValues()));
		end
	end
end

--Function for adding a roller to an array.
local function addRollerToArray(array, roller)
	--Determine if user has already rolled, and if found then exit function.
	if( # array > 0) then
		for i,v in ipairs(array) do 
			if(v[1] == roller[1]) then
				return;
			end
		end
	end
	
	--Clear the current frame and add the roller to the list.
	table.insert(array, roller);
end

--Function for handling history list displays.
local function scrollHistory(bool)
	--Check if we're trying to scroll up or down.
	if bool then
		if(# rollHistory >= historyIndex + 1) then 
			historyIndex = historyIndex + 1;
		end
	else
		if(0 < historyIndex - 1) then 
			historyIndex = historyIndex - 1;
		end
	end
	
	--Check if there's more history left and disable/enable buttons accordingly.
	if(# rollHistory >= historyIndex +1) then 
		raidLeadConfig.historyButtonFrame.nextHistoryPage:Enable();
	else
		raidLeadConfig.historyButtonFrame.nextHistoryPage:Disable();
	end
	if(0 < historyIndex -1) then 
		raidLeadConfig.historyButtonFrame.previousHistoryPage:Enable();
	else
		raidLeadConfig.historyButtonFrame.previousHistoryPage:Disable();
	end
	
	raidLeadConfig.historyButtonFrame.pageNumber:SetText(historyIndex .. "/" .. # rollHistory);
	
	--If historyIndex is 0 that means we are at current item and should display that. Otherwise display item at place historyIndex.
	local id = rollHistory[historyIndex].item:match("item:(%d+):");
	setItemButtonData(raidLeadConfig.itemButton, id, rollHistory[historyIndex].item);
		
	updateMessageframe(raidLeadConfig.mainFrame.messageFrame, rollHistory[historyIndex].mainList);
	updateMessageframe(raidLeadConfig.offFrame.messageFrame, rollHistory[historyIndex].offList);
	updateMessageframe(raidLeadConfig.mogFrame.messageFrame, rollHistory[historyIndex].mogList);
	updateMessageframe(raidLeadConfig.otherFrame.messageFrame, rollHistory[historyIndex].otherList);
end

--Function for handling the manual rolls.
local function manualRoll(self)
	local editBox = self.editBox;
	local text = editBox:GetText();
	local number = tonumber(text);
	if not (number == 0 or number == "" or number == nil) then
		if(userConfig:IsShown()) then
			userConfig:Hide();
		else
			toggleButtons(raidLeadConfig.rollFrame, false);
		end
		RandomRoll(1,number);
	end
	self:Hide()
end

--Custom printout method for yellow message printouts.
local function printOut(msg)
	DEFAULT_CHAT_FRAME:AddMessage(msg,1,1,0);
end

-------------
--FACTORIES--
-------------

--Button factory.
local function startButton(button, frame,  xValue, yValue, title, xLength, yLength, func)
	button:SetPoint("CENTER", frame, "TOP", xValue, yValue);
	button:SetSize(xLength,yLength);
	button:SetText(title);
	button:SetNormalFontObject(functions.fontNormalObject);
	button:SetHighlightFontObject(functions.fontHighlightObject);
	button:SetScript("OnClick", func);
end

--Roll list factory.
local function startFrame(frame, config, xValue, yValue, title, length)
	frame.width  = length;
	frame.height = 130;
	frame:SetSize(frame.width, frame.height);
	frame:SetPoint("LEFT", config, "LEFT", xValue, yValue);
	frame:EnableMouse(true);
	frame:EnableMouseWheel(true);
	
	frame.title = frame:CreateFontString(nil,"OVERLAY");
	frame.title:SetFontObject(functions.fontHighlightObject);
	frame.title:SetPoint("TOP", frame, "TOP", 0, 1);
	frame.title:SetText(title);
	
	--ScrollingMessageFrame
	local messageFrame = CreateFrame("ScrollingMessageFrame", nil, frame);
	messageFrame:SetPoint("TOPLEFT", 0, -15);
	messageFrame:SetPoint("BOTTOMRIGHT", 0, 40);
	messageFrame:SetFontObject(functions.fontNormalObject);
	messageFrame:SetTextColor(1, 1, 1, 1);
	messageFrame:SetJustifyH("LEFT");
	messageFrame:SetHyperlinksEnabled(true);
	messageFrame:SetFading(false);
	messageFrame:SetMaxLines(300);
	frame.messageFrame = messageFrame;
	
	
	local tex = messageFrame:CreateTexture(nil,"BACKGROUND");
	tex:SetAllPoints();
	tex:SetColorTexture(0, 0, 0);
	local backDrop = CreateFrame("Frame", nil, messageFrame);
	backDrop:SetPoint("TOPLEFT", -5, 6);
	backDrop:SetPoint("BOTTOMRIGHT", 2, -6);

	backDrop:SetBackdrop( 
	{ 
		bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
		edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
		tile = true, tileSize = 16, edgeSize = 16, 
		insets = { left = 4, right = 4, top = 4, bottom = 4 }
	});										
	backDrop:SetBackdropColor(0,0,0,0);
	
	---------------
	-- Scroll bar--
	---------------
	local scrollBar = CreateFrame("Slider", nil, frame, "UIPanelScrollBarTemplate");
	scrollBar:SetPoint("RIGHT", frame, "RIGHT", 4, 11);
	scrollBar:SetSize(30, frame.height - 85);
	scrollBar:SetMinMaxValues(0, 0);
	scrollBar:SetValueStep(1);
	scrollBar:SetObeyStepOnDrag(true);
	scrollBar.scrollStep = 1;
	frame.scrollBar = scrollBar;

	scrollBar:SetScript("OnValueChanged", function(self, value)
		messageFrame:SetScrollOffset(select(2, scrollBar:GetMinMaxValues()) - value);
	end);
	
	scrollBar:SetValue(0);

	frame:SetScript("OnMouseWheel", function(self, delta)
		local cur_val = scrollBar:GetValue();
		local min_val, max_val = scrollBar:GetMinMaxValues();

		if delta < 0 and cur_val < max_val then
			cur_val = math.min(max_val, cur_val + 1);
			scrollBar:SetValue(cur_val);
		elseif delta > 0 and cur_val > min_val then
			cur_val = math.max(min_val, cur_val - 1);
			scrollBar:SetValue(cur_val);
		end
	end)
end

--------------------------------------------
-----------------MAIN WINDOW----------------
--------------------------------------------
userConfig:SetFrameStrata("LOW");
userConfig:SetSize(200,100);
userConfig:SetScale(functions.calculateScale());
userConfig:SetMovable(true);
userConfig:EnableMouse(true);
userConfig:RegisterForDrag("LeftButton");
userConfig:SetScript("OnDragStart", userConfig.StartMoving)
userConfig:SetScript("OnDragStop", userConfig.StopMovingOrSizing)
userConfig.title = userConfig:CreateFontString(nil,"OVERLAY");
userConfig.title:SetFontObject(functions.fontHighlightObject);
userConfig.title:SetPoint("LEFT", userConfig.TitleBg, "left", 5, 0);
userConfig.title:SetText("Loot Roll User");
userConfig:SetClampedToScreen(true);
userConfig:SetClampRectInsets(0, 0, 0, 0);
userConfig:Hide();

--MAIN WINDOW ITEM PREVIEW
userConfig.itemButton = CreateFrame("Button", nil, userConfig, "GameMenuButtonTemplate");
userConfig.itemButton:SetPoint("LEFT", userConfig, "LEFT", 10, -10);
userConfig.itemButton:SetSize(60,60);
userConfig.itemButton:SetScript("OnLeave", function(self) GameTooltip:Hide() ResetCursor() end);

userConfig.itemButton.itemTexture = userConfig.itemButton:CreateTexture(nil,"OVERLAY");
userConfig.itemButton.itemTexture:SetPoint("TOPLEFT", 0, 0);
userConfig.itemButton.itemTexture:SetPoint("BOTTOMRIGHT", 0, 0);

--MAIN WINDOW BUTTONS
userConfig.mainSpeccButton = CreateFrame("Button", nil, userConfig, "GameMenuButtonTemplate");
userConfig.offSpeccButton = CreateFrame("Button", nil, userConfig, "GameMenuButtonTemplate");
userConfig.mogButton = CreateFrame("Button", nil, userConfig, "GameMenuButtonTemplate");
userConfig.otherButton = CreateFrame("Button", nil, userConfig, "GameMenuButtonTemplate");

--START INSTANCE OF MAIN WINDOW BUTTONS
startButton(userConfig.mainSpeccButton, userConfig, 0, -45, "Main", 60, 30, function(self) RandomRoll(1,llrSettings.mainRoll) userConfig:Hide() end);
startButton(userConfig.offSpeccButton, userConfig, 0, -75, "Off", 60, 30, function(self) RandomRoll(1,llrSettings.offRoll) userConfig:Hide() end);
startButton(userConfig.mogButton, userConfig, 60, -45, "Mog", 60, 30, function(self) RandomRoll(1,llrSettings.mogRoll) userConfig:Hide() end);
startButton(userConfig.otherButton, userConfig, 60, -75, "Other", 60, 30, function(self) StaticPopup_Show("ManualRolling") end);

--------------------------------------------
------------RAID LEADER WINDOW--------------
--------------------------------------------
raidLeadConfig:SetFrameStrata("MEDIUM");
raidLeadConfig:SetSize(340,235);
raidLeadConfig:SetScale(functions.calculateScale());
raidLeadConfig:SetMovable(true);
raidLeadConfig:EnableMouse(true);
raidLeadConfig:RegisterForDrag("LeftButton");
raidLeadConfig:SetScript("OnDragStart", function(self) 
	raidLeadConfig:StartMoving();
end);
raidLeadConfig:SetScript("OnDragStop", function(self) raidLeadConfig:StopMovingOrSizing() end);
raidLeadConfig:SetClampedToScreen(true);
raidLeadConfig:SetClampRectInsets(0, 0, 0, 0)
raidLeadConfig:SetScript("OnShow", function(self) 
	llr_EventFrame:RegisterEvent("CHAT_MSG_SYSTEM");
end);
raidLeadConfig:SetScript("OnHide", function(self) llr_EventFrame:UnregisterEvent("CHAT_MSG_ADDON"); end);
raidLeadConfig:SetScript("OnMouseWheel", function(self, delta)
	if # rollHistory == 0 then return end;
	if (delta < 0) and # rollHistory >= historyIndex + 1 then
		scrollHistory(true);
	elseif (delta > 0) and 0 <= historyIndex - 1 then
		scrollHistory(false);
	end
end);
raidLeadConfig.title = raidLeadConfig:CreateFontString(nil,"OVERLAY");
raidLeadConfig.title:SetFontObject(functions.fontHighlightObject);
raidLeadConfig.title:SetPoint("LEFT", raidLeadConfig.TitleBg, "LEFT", 5, 0);
raidLeadConfig.title:SetText("Loot Roll Leader");
raidLeadConfig:Hide();

--Setup the raid leader item preview.
raidLeadConfig.itemButton = CreateFrame("Button", nil, raidLeadConfig, "GameMenuButtonTemplate");
raidLeadConfig.itemButton:SetPoint("TOPLEFT", raidLeadConfig, "TOPLEFT", 10, -42);
raidLeadConfig.itemButton:SetSize(60,60);
raidLeadConfig.itemButton:SetScript("OnLeave", function(self) GameTooltip:Hide() ResetCursor() end);

raidLeadConfig.itemButton.itemTexture = raidLeadConfig.itemButton:CreateTexture(nil,"OVERLAY");
raidLeadConfig.itemButton.itemTexture:SetPoint("TOPLEFT", 0, 0);
raidLeadConfig.itemButton.itemTexture:SetPoint("BOTTOMRIGHT", 0, 0);

--Initialize raid leader's roll buttons.
raidLeadConfig.rollFrame = CreateFrame("Frame","RollButtons", raidLeadConfig);
raidLeadConfig.rollFrame:SetSize(60,120);
raidLeadConfig.rollFrame:SetPoint("LEFT", raidLeadConfig, "LEFT", 30, 10);

raidLeadConfig.rollFrame.mainSpeccButton = CreateFrame("Button", nil, raidLeadConfig.rollFrame, "GameMenuButtonTemplate");
raidLeadConfig.rollFrame.offSpeccButton = CreateFrame("Button", nil, raidLeadConfig.rollFrame, "GameMenuButtonTemplate");
raidLeadConfig.rollFrame.mogButton = CreateFrame("Button", nil, raidLeadConfig.rollFrame, "GameMenuButtonTemplate");
raidLeadConfig.rollFrame.otherButton = CreateFrame("Button", nil, raidLeadConfig.rollFrame, "GameMenuButtonTemplate");

--Start instance of raid leader buttons.
startButton(raidLeadConfig.rollFrame.mainSpeccButton, raidLeadConfig.rollFrame, -20, -73, "Main", 60, 30, function(self) RandomRoll(1,llrSettings.mainRoll) toggleButtons(raidLeadConfig.rollFrame, false)  end);
startButton(raidLeadConfig.rollFrame.offSpeccButton, raidLeadConfig.rollFrame, -20, -103, "Off", 60, 30, function(self) RandomRoll(1,llrSettings.offRoll) toggleButtons(raidLeadConfig.rollFrame, false) end);
startButton(raidLeadConfig.rollFrame.mogButton, raidLeadConfig.rollFrame, -20, -133, "Mog", 60, 30, function(self) RandomRoll(1,llrSettings.mogRoll) toggleButtons(raidLeadConfig.rollFrame, false) end);
startButton(raidLeadConfig.rollFrame.otherButton, raidLeadConfig.rollFrame, -20, -163, "Other", 60, 30, function(self) StaticPopup_Show("ManualRolling") end);

--Initalize history buttons
raidLeadConfig.historyButtonFrame = CreateFrame("Frame","HistoryButtons", raidLeadConfig);
raidLeadConfig.historyButtonFrame:SetSize(20,15);
raidLeadConfig.historyButtonFrame:SetPoint("TOPRIGHT", raidLeadConfig, "TOPRIGHT", -25, -10);

raidLeadConfig.historyButtonFrame.pageNumber = raidLeadConfig:CreateFontString(nil,"OVERLAY");
raidLeadConfig.historyButtonFrame.pageNumber:SetFontObject(functions.fontHighlightObject);
raidLeadConfig.historyButtonFrame.pageNumber:SetPoint("LEFT", raidLeadConfig.historyButtonFrame, "LEFT", -55, 5);
raidLeadConfig.historyButtonFrame.pageNumber:SetText("1/1");

raidLeadConfig.historyButtonFrame.nextHistoryPage = CreateFrame("Button", nil, raidLeadConfig.historyButtonFrame, "GameMenuButtonTemplate");
raidLeadConfig.historyButtonFrame.previousHistoryPage = CreateFrame("Button", nil, raidLeadConfig.historyButtonFrame, "GameMenuButtonTemplate");

raidLeadConfig.historyButtonFrame.nextHistoryPage:Disable();
raidLeadConfig.historyButtonFrame.previousHistoryPage:Disable();

--Start instance of history buttons
startButton(raidLeadConfig.historyButtonFrame.nextHistoryPage, raidLeadConfig.historyButtonFrame, 0, 0, ">", 25, 20, function(self) scrollHistory(true) end);
startButton(raidLeadConfig.historyButtonFrame.previousHistoryPage, raidLeadConfig.historyButtonFrame, -22, 0, "<", 25, 20, function(self) scrollHistory(false) end);

--Initialize the frames for the rolled lists.
raidLeadConfig.mainFrame = CreateFrame("Frame", "MainList", raidLeadConfig);
raidLeadConfig.offFrame = CreateFrame("Frame", "OffList", raidLeadConfig);
raidLeadConfig.mogFrame = CreateFrame("Frame", "MogList", raidLeadConfig);
raidLeadConfig.otherFrame = CreateFrame("Frame", "OtherList", raidLeadConfig);

--Start the instance of the rolled list frames.
startFrame(raidLeadConfig.mainFrame, raidLeadConfig, 77, 21, "Main", 120);
startFrame(raidLeadConfig.offFrame, raidLeadConfig, 77, -77, "Off", 120);
startFrame(raidLeadConfig.mogFrame, raidLeadConfig, 207, 21, "Mog", 120);
startFrame(raidLeadConfig.otherFrame, raidLeadConfig, 207, -77, "Other (Max: 9999)", 120);

--Pop up for manual rolls.
StaticPopupDialogs["ManualRolling"] = 
{
	text = "Enter roll (Max: 9999):",
	button1 = "Accept",
	button2 = "Cancel",
	hasEditBox = 1,
	maxLetters = 4,
	OnShow = function(self)
		local editBox = self.editBox;
		editBox:SetText("");
		editBox:SetFocus();
	end,
	OnAccept = function(self)
		manualRoll(self);
	end,
	EditBoxOnEnterPressed = function(self)
		manualRoll(self:GetParent());
	end,
	EditBoxOnEscapePressed = function(self)
		self:GetParent():Hide();
	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1
};

--MAIN CODE--
--Slash commands for manual control.
SLASH_LLR1 = "/llr"
SLASH_LLR2 = "/lptlootroll"

local itemTestArr = 
{
	"item:160690::::::::120:253::6:1:3524:::",
	"item:160679::::::::120:253::6:1:3524:::",
	"item:160689::::::::120:253::6:1:3524:::",
	"item:160688::::::::120:253::6:1:3524:::",
	"item:160686::::::::120:253::6:1:3524:::",
	"item:160685::::::::120:253::6:1:3524:::",
	"item:160687::::::::120:253::6:1:3524:::",
	"item:160681::::::::120:253::6:1:3524:::",
	"item:160693::::::::120:253::6:1:3524:::",
	"item:160683::::::::120:253::6:1:3524:::",
	"item:160684::::::::120:253::6:1:3524:::",
	"item:160678::::::::120:253::6:1:3524:::",
	"item:160694::::::::120:253::6:1:3524:::",
	"item:160680::::::::120:253::6:1:3524:::",
	"item:160698::::::::120:253::6:1:3524:::",
	"item:160696::::::::120:253::6:1:3524:::",
	"item:160695::::::::120:253::6:1:3524:::",
	"item:163596::::::::120:253::6:1:3524:::",
	"item:160630::::::::120:253::6:1:3524:::",
	"item:160634::::::::120:253::6:1:3524:::",
	"item:160623::::::::120:253::6:1:3524:::",
	"item:160616::::::::120:253::6:1:3524:::",
	"item:160719::::::::120:253::6:1:3524:::",
	"item:160726::::::::120:253::6:1:3524:::",
	"item:160632::::::::120:253::6:1:3524:::",
	"item:160613::::::::120:253::6:1:3524:::",
	"item:160725::::::::120:253::6:1:3524:::",
	"item:160627::::::::120:253::6:1:3524:::",
	"item:160721::::::::120:253::6:1:3524:::"
};

SlashCmdList.LLR = function(msg)
	local msg = string.lower(msg);
	local array = functions.stringSplit(msg, " ");
	local link = nil;
	
	if(array[2] == "test") then
		local randomIndex = math.random(1,# itemTestArr);
		_,link = GetItemInfo(itemTestArr[randomIndex]);
		
		if(link == nil) then 
			print("Nil found at index: " .. randomIndex .. " item returned was: " .. itemTestArr[randomIndex]);
			print("Error most likely caused by GetItemInfo being slow, please try again!");
			return;
		end
		
		table.insert(rollHistory, 1, 
		{
			item 		= link,
			mainList 	= {},
			offList 	= {},
			mogList 	= {},
			otherList 	= {}
		});
		historyLengthControl();
		
		local id = rollHistory[1].item:match("item:(%d+):");
		
		print("The current scale is: " .. functions.calculateScale());
		clearMessageFrames();
		setItemButtonData(userConfig.itemButton, id, rollHistory[1].item);
		setItemButtonData(raidLeadConfig.itemButton, id, rollHistory[1].item);
		historyIndex = 1;
		raidLeadConfig.historyButtonFrame.previousHistoryPage:Disable();
		
		if(# rollHistory > 1) then
			raidLeadConfig.historyButtonFrame.pageNumber:SetText(historyIndex .. "/" .. # rollHistory);
			raidLeadConfig.historyButtonFrame.nextHistoryPage:Enable();
		end
	end
	
	if(array[1] == "leader") then
		--Toggle raidLeadConfig UI.
		if not raidLeadConfig:IsShown() then
			raidLeadConfig:Show();
		end 
		
		updateListNames();
		
		if llrSettings.saveMode and rollHistory[1] then 
			scrollHistory(false);
		end
		
		if(array[2] == "test") then 
			local iMax = tonumber(array[3]);
			
			if (iMax == 0 or iMax == "" or iMax == nil) then
				iMax = 15;
			end
			
			printOut("Test populated with " .. rollHistory[1].item .. " and ".. iMax .. " rollers.");
			toggleButtons(raidLeadConfig.rollFrame, true);
			
			for i=1,iMax,1 do 
				local testName = randomString(12):gsub("^%l", string.upper);
				local color1 = math.random(20,100)/100;
				local color2 = math.random(20,100)/100;
				local color3 = math.random(20,100)/100;
				
				addRollerToArray(rollHistory[1].mainList, {testName, math.random(1,llrSettings.mainRoll),color1,color2,color3})
				addRollerToArray(rollHistory[1].offList, {testName, math.random(1,llrSettings.offRoll),color1,color2,color3})
				addRollerToArray(rollHistory[1].mogList, {testName, math.random(1,llrSettings.mogRoll),color1,color2,color3})
				addRollerToArray(rollHistory[1].otherList, {testName, math.random(1,9999),color1,color2,color3})
			end
			
			updateMessageframe(raidLeadConfig.mainFrame.messageFrame, rollHistory[1].mainList);
			updateMessageframe(raidLeadConfig.offFrame.messageFrame, rollHistory[1].offList);
			updateMessageframe(raidLeadConfig.mogFrame.messageFrame, rollHistory[1].mogList);
			updateMessageframe(raidLeadConfig.otherFrame.messageFrame, rollHistory[1].otherList);
		elseif(array[2] == "getpos") then
			point, relativeTo, relativePoint, xOfs, yOfs = raidLeadConfig:GetPoint();
			print(point, relativeTo, relativePoint, xOfs, yOfs);
		end
		return;
	elseif(array[1] == "user") then
		--Toggle userConfig UI.
		if not userConfig:IsShown() then
			userConfig:Show();
		end
		return;
	elseif(array[1] == "clear") then
		clearMessageFrames();
		raidLeadConfig.historyButtonFrame.pageNumber:SetText("1/1");
		raidLeadConfig.historyButtonFrame.nextHistoryPage:Disable();
		raidLeadConfig.historyButtonFrame.previousHistoryPage:Disable();
		for i=1, #rollHistory do 
			rollHistory[i]=nil 
		end;
	elseif(array[1] == "reset") then
		printOut("Location of windows have been reset.");
		userConfig:ClearAllPoints();
		raidLeadConfig:ClearAllPoints();
		userConfig:SetPoint("CENTER", UIParent, "CENTER");
		raidLeadConfig:SetPoint("CENTER", UIParent, "CENTER");
		return;
	elseif(array[1] == "msg") then
		C_ChatInfo.SendAddonMessage("lptlootroll", llrSettings.mainRoll .. "-" .. llrSettings.offRoll .. "-" .. llrSettings.mogRoll, "RAID");
		return;
	elseif(array[1] == "config" or array[1] == "settings") then
		functions.toggleLlrConfig();
		return;
	elseif(array[1] == "scale") then
		local testScale = tonumber(array[2]);
		if(testScale == 0 or testScale == "" or testScale == nil) then
			testScale = 1;
		end
		local scale = functions.calculateScale();
		raidLeadConfig:SetScale(scale*testScale);
		userConfig:SetScale(scale*testScale);
		print("Scale set to: " .. scale*testScale);
		return;
	end 
	print("");
	printOut(GetAddOnMetadata("LPT Loot Roll", "Title") .. " version: " .. GetAddOnMetadata("LPT Loot Roll", "Version"));
	if(array[1] == nil or array[1] == "") then  
		printOut("No command recieved.");
	else
		printOut("Error: Command not recognized.");
	end
	printOut("Avaiable commands are: \n /llr leader - displays raidleader window. \n /llr user - displays the normal user window. \n /llr reset - resets position of windows. \n /llr config - open the config menu \n /llr clear - clear all stored roll data");
	print("");
end

llr_EventFrame:RegisterEvent("CHAT_MSG_RAID_WARNING");
llr_EventFrame:SetScript("OnEvent", function(self,event,...)
	if(event == "CHAT_MSG_RAID_WARNING") then
		local arg1, arg2 = ...;
		local name = functions.stringSplit(arg2, "-")[1];
		
		--Check if the person sending a raidwarning is the raid leader.
		if (UnitIsGroupLeader(name)) then
			local _,playerClass = UnitClass("Player");
			local _,link,_,_,_,itemType,itemSubTybe,_,invType = GetItemInfo(arg1);
			if not (link == nil or link == "") and (itemType == "Armor" or itemType == "Weapon") then
				--Class array to store what each class can equip.
				local classArray = 
				{
					DEATHKNIGHT = {"Plate","One-Handed Axes","Two-Handed Axes","One-Handed Swords","Two-Handed Swords","One-Handed Maces", "Two-Handed Maces","Polearms"},
					DEMONHUNTER = {"Leather","Warglaives","Fist Weapons", "One-Handed Axes", "One-Handed Swords"},
					DRUID = 	  {"Leather","One-Handed Maces","Two-Handed Maces","Polearms","Staves","Daggers","Fist Weapons"},
					HUNTER = 	  {"Mail","One-Handed Axes","Two-Handed Axes","One-Handed Swords","Two-Handed Swords","Polearms","Staves","Daggers","Fist Weapons","Bows","Crossbows","Guns"},
					MAGE = 		  {"Cloth","One-Handed Swords","Staves","Daggers","Wands"},
					MONK = 		  {"Leather","One-Handed Axes","One-Handed Swords","One-Handed Maces","Polearms","Staves","Fist Weapons"},
					PALADIN = 	  {"Plate","Shield","One-Handed Axes","Two-Handed Axes","One-Handed Swords","Two-Handed Swords","One-Handed Maces","Two-Handed Maces", "Polearms"},
					PRIEST = 	  {"Cloth","One-Handed Maces","Staves","Daggers","Wands"},
					ROGUE = 	  {"Leather","One-Handed Axes","One-Handed Swords","One-Handed Maces","Daggers","Fist Weapons","Bows","Crossbows","Guns"},
					SHAMAN = 	  {"Mail","Shield","One-Handed Axes","Two-Handed Axes","One-Handed Maces","Two-Handed Maces","Staves","Daggers","Fist Weapons"},
					WARLOCK = 	  {"Cloth","One-Handed Swords","Staves","Daggers","Wands"},
					WARRIOR = 	  {"Plate","Shield","One-Handed Axes","Two-Handed Axes","One-Handed Swords","Two-Handed Swords","One-Handed Maces","Two-Handed Maces","Polearms","Staves","Daggers","Fist Weapons","Bows","Crossbows","Guns"},
				};
				
				--Determine if the player's class can equip the item.
				local boolean = false;
				for i,v in ipairs(classArray[playerClass]) do
					if(v==itemSubTybe) then
						boolean = true;
						break;
					end
				end
				
				table.insert(rollHistory, 1, 
				{
					item 		= link,
					mainList 	= {},
					offList 	= {},
					mogList 	= {},
					otherList   = {}
				});
				historyLengthControl();
				
				--Get the ID from the item link.
				local id = rollHistory[1].item:match("item:(%d+):");
				
				--Determine the equipable boolean. Check if it's an equipable type or if it's subtype is misc (neck, trinket, ring) or if the item is a cloak.
				local isEquipable = boolean or itemSubTybe == "Miscellaneous" or invType == "INVTYPE_CLOAK";
				
				if UnitIsGroupLeader("Player") or llrSettings.leadMode then
					--Setup the item "frame" with tooltip, mog and artifact system implementation.
					setItemButtonData(raidLeadConfig.itemButton, id, rollHistory[1].item);
					
					--Clear all roll frames and lists.
					clearMessageFrames();
					historyIndex = 1;
					
					--Prepare history buttons.
					raidLeadConfig.historyButtonFrame.previousHistoryPage:Disable();
					if(# rollHistory > 1) then
						raidLeadConfig.historyButtonFrame.pageNumber:SetText(historyIndex .. "/" .. # rollHistory);
						raidLeadConfig.historyButtonFrame.nextHistoryPage:Enable();
					end
					
					--Show raid leader UI.
					raidLeadConfig:Show();
					updateListNames();
					
					--If leader can equip, enable buttons, otherwise disable.
					if isEquipable then
						toggleButtons(raidLeadConfig.rollFrame, true);
					else
						toggleButtons(raidLeadConfig.rollFrame, false);
					end
				else
					if isEquipable then
						--Setup the item "frame" with tooltip, mog and artifact system implementation.
						setItemButtonData(userConfig.itemButton, id, rollHistory[1].item);
						userConfig:Show();
					end
				end
			end 
		end
	elseif(event == "CHAT_MSG_SYSTEM" and raidLeadConfig:IsShown()) then
		local message = ...;
		local author, rollResult, rollMin, rollMax = string.match(message, "(.+) rolls (%d+) %((%d+)-(%d+)%)");
		if author then
			--print(author .. " rolled: " .. rollResult .. ". With roll: " .. rollMin .. "-" .. rollMax);
			local _,playerClass = UnitClass(author);
			local color = RAID_CLASS_COLORS[playerClass];			
			
			--Logic to determine the roll and to then fill in the appropiate list with the roll.
			local rolled = tonumber(rollResult);
			local rolledMax = tonumber(rollMax);
			if not (rolled == 0 or rolled == "" or rolled == nil or rolledMax == 0 or rolledMax == "" or rolledMax == nil) then
				local roller = {author, rolled, color.r, color.g, color.b};
				if(rolledMax == llrSettings.mainRoll) then
					addRollerToArray(rollHistory[1].mainList, roller);
				elseif (rolledMax == llrSettings.offRoll) then
					addRollerToArray(rollHistory[1].offList, roller);
				elseif (rolledMax == llrSettings.mogRoll) then
					addRollerToArray(rollHistory[1].mogList, roller);
				else
					--Check that rolledMax is less than 10 000 (to match editbox of other roll).
					if (rolledMax < 10000) then
						roller[2] = rolledMax;
						addRollerToArray(rollHistory[1].otherList, roller);
					end
				end
				if(historyIndex == 1) then
					updateMessageframe(raidLeadConfig.mainFrame.messageFrame, rollHistory[1].mainList);
					updateMessageframe(raidLeadConfig.offFrame.messageFrame, rollHistory[1].offList);
					updateMessageframe(raidLeadConfig.mogFrame.messageFrame, rollHistory[1].mogList);
					updateMessageframe(raidLeadConfig.otherFrame.messageFrame, rollHistory[1].otherList);
				end
			end
		end
	end 
end);